package com.qmth.cqb.paper.service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.qmth.cqb.utils.exception.PaperException;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.wml.P;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.gson.Gson;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.model.ImportPaperCheck;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.model.QuestionType;
import com.qmth.cqb.question.service.QuesService;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.PaperStatus;
import com.qmth.cqb.utils.enums.PaperType;
import com.qmth.cqb.utils.enums.QuesUnit;
import com.qmth.cqb.utils.word.DocxProcessUtil;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

@Service
public class ImportPaperService {

    protected static final Logger log = LoggerFactory.getLogger(ImportPaperService.class);

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;

    @Autowired
    PaperDetailUnitRepo paperDetailUnitRepo;

    @Autowired
    QuesService quesService;

    @Autowired
    Gson gson;

    @Autowired
    PaperDetailService paperDetailService;

    @Autowired
    PaperService paperService;

    /**
     * 获取上传文件
     *
     * @param file
     * @return
     */
    public File getUploadFile(CommonsMultipartFile file) throws Exception{
        String fileName = file.getOriginalFilename();
        File tempFile = new File(DocxProcessUtil.TEMP_FILE_IMP + fileName);
        OutputStream os = null;
        os = new FileOutputStream(tempFile);
        IOUtils.copyLarge(file.getInputStream(), os);
        IOUtils.closeQuietly(os);
        return tempFile;
    }

    /**
     * 导入试卷
     *
     * @param paperName
     * @param courseNo
     * @param file
     * @return
     */
    public String ImportPaper(String paperName,
                              String courseNo,
                              String courseName,
                              AccessUser user,
                              File file) throws Exception{
        String errorInfo = paperService.checkPaperName(paperName, user.getOrgId().toString());
        if (errorInfo == null) {
            errorInfo = "";
            errorInfo += processImportPaper(paperName, courseNo, courseName, user, file);
            if (StringUtils.isEmpty(errorInfo)) {
                return "success";
            } else {
                log.error("导入异常:" + errorInfo);
                return errorInfo;
            }
        } else {
            return errorInfo;
        }
    }

    /**
     * 保存导入试卷信息
     *
     * @param paper
     * @param paperDetails
     * @param paperDetailUnits
     * @param importPaperCheck
     */
    public void savePaper(Paper paper, List paperDetails, List paperDetailUnits, List questions,
            ImportPaperCheck importPaperCheck) {

        Paper tempPaper = null;

        if (!StringUtils.isEmpty(importPaperCheck.errorInfo))
            return;

        if (paper != null) {
            tempPaper = paperRepo.save(paper);
        }

        if (paperDetails.size() > 0) {
            paperDetailRepo.save(paperDetails);
        }

        if (questions.size() > 0) {
            quesService.saveImportQuesList(questions);
        }

        if (paperDetailUnits.size() > 0) {
            paperDetailUnitRepo.save(paperDetailUnits);
        }

        paperService.formatPaper(tempPaper, null);
    }

    /**
     * 处理导入试卷
     * 
     * @param paperName
     * @param courseNo
     * @param courseName
     * @param user
     * @param file
     * @return
     */
    public String processImportPaper(String paperName, String courseNo, String courseName, AccessUser user, File file) throws Exception {

        WordprocessingMLPackage wordMLPackage;
        WordprocessingMLPackage tmpWordMlPackage;
        WordprocessingMLPackage writePkg;
        ImportPaperCheck importPaperCheck = new ImportPaperCheck();

        wordMLPackage = WordprocessingMLPackage.load(file);
        // 初始化图片路径
        DocxProcessUtil.initPkgImage(wordMLPackage);
        // 深拷贝临时pkg与最终写入数据库pkg
        tmpWordMlPackage = DocxProcessUtil.getTmpPackage(wordMLPackage);
        writePkg = DocxProcessUtil.getTmpPackage(wordMLPackage);

        // 获取word文档中所有段落
        List<Object> pList = DocxProcessUtil.getAllElementFromObject(wordMLPackage.getMainDocumentPart(), P.class);

        // 创建试卷类
        Paper paper = new Paper();

        paper.setCourseNo(courseNo);

        paper.setCourseName(courseName);

        paper.setOrgId(user.getOrgId().toString());

        paper.setCreator(user.getName());

        // 设置试卷
        initPaper(paper, paperName);

        // 创建空大题类
        PaperDetail paperDetail = null;

        // 创建大题集合
        List<PaperDetail> paperDetails = new ArrayList<PaperDetail>();

        // 创建小题集合
        List<PaperDetailUnit> paperDetailUnits = new ArrayList<PaperDetailUnit>();

        // 创建试题集合
        List<Question> questions = new ArrayList<Question>();

        // 大题号
        int mainQuesNum = 0;

        // 小题号
        int subQuesNum = 0;

        for (int i = 0; i < pList.size(); i++) {
            P p = (P) pList.get(i);
            String pText = DocxProcessUtil.getPText(p);


            if (StringUtils.isEmpty(pText)) {
                // 跳过空白段落
                continue;

            } else if (pText.startsWith("[试题分类]")) {
                // 处理大题头信息
                processMainQuesHeader(pList, importPaperCheck.index, importPaperCheck);

                // 校验大题头信息
                if (!checkQuesHeader(importPaperCheck)) {
                    throw new PaperException(importPaperCheck.errorInfo);
                }

                // 创建大题类
                paperDetail = new PaperDetail();

                // 设置大题类
                initQuesHeader(paper, paperDetail, paperDetails, ++mainQuesNum, importPaperCheck);

            } else if (pText.matches("^\\d{1,}\\.[\\s\\S]*")
                    || (isNested(importPaperCheck) && !pText.startsWith("["))) {
                // 处理试题

                // 创建小题类和试题类
                PaperDetailUnit paperDetailUnit = new PaperDetailUnit();
                Question question = new Question();

                // 初始化小题类和试题类
                initPaperDetail(paper, paperDetail, paperDetailUnit, question, ++subQuesNum, importPaperCheck);

                // 处理客观题
                if (importPaperCheck.quesType.equals("单选") || importPaperCheck.quesType.equals("多选")) {

                    // 处理题干
                    processQuesBody(pList, importPaperCheck.index, question, importPaperCheck, tmpWordMlPackage);
                    // 处理选项
                    processQuesOption(pList, importPaperCheck.index, subQuesNum, question, importPaperCheck,
                            tmpWordMlPackage);
                    // 处理尾信息
                    processQuesTail(pList, importPaperCheck.index, subQuesNum, question, paperDetailUnit,
                            importPaperCheck, tmpWordMlPackage, false);
                    // 处理选择题的option--chenken 20170425
                    processSelectOption(question);
                } else if (importPaperCheck.quesType.equals("套题")) {
                    // 处理套题
                    processNestedQues(pList, importPaperCheck.index, question, paperDetailUnit, importPaperCheck,
                            tmpWordMlPackage);
                } else {
                    // 处理其他题型
                    processQuesBody(pList, importPaperCheck.index, question, importPaperCheck, tmpWordMlPackage);
                    processQuesTail(pList, importPaperCheck.index, subQuesNum, question, paperDetailUnit,
                            importPaperCheck, tmpWordMlPackage, false);
                }
                // 设置WordMlPackage二进制数据
                setPkgByte(question, writePkg);
                // 设置question与Unit集合数据
                question.setCourseNo(paper.getCourseNo());
                question.setCourseName(paper.getCourseName());
                question.setOrgId(user.getOrgId().toString());
                questions.add(question);
                paperDetailUnits.add(paperDetailUnit);

                // 设置当前索引,防止多余循环
                i = importPaperCheck.index - 1;
            }
            if (!StringUtils.isEmpty(importPaperCheck.errorInfo)) {
                throw new PaperException(importPaperCheck.errorInfo);
            }
        }
        if (paperDetails.size() == 0) {
            importPaperCheck.setErrorInfo("导入文件格式有误!");
            throw new PaperException("导入文件格式有误!");
        }
        paper.setPaperDetailCount(mainQuesNum);
        // 保存导入试卷信息
        savePaper(paper, paperDetails, paperDetailUnits, questions, importPaperCheck);
        wordMLPackage = null;
        tmpWordMlPackage = null;
        writePkg = null;
        FileUtils.deleteQuietly(file);
        return importPaperCheck.errorInfo;
    }

    /**
     * 初始化试卷信息
     *
     * @param paper
     * @param paperName
     */
    public void initPaper(Paper paper, String paperName) {
        paper.setName(paperName);
        paper.setTitle(paperName);
        paper.setPaperType(PaperType.IMPORT);
        paper.setPaperStatus(PaperStatus.DRAFT);
    }

    /**
     * 初始化大题头信息
     *
     * @param paper
     * @param paperDetail
     * @param paperDetails
     * @param mainQuesNum
     * @param importPaperCheck
     */
    public void initQuesHeader(Paper paper, PaperDetail paperDetail, List<PaperDetail> paperDetails, int mainQuesNum,
            ImportPaperCheck importPaperCheck) {

        paperDetail.setPaper(paper);
        // 设置大题信息
        paperDetail.setName(importPaperCheck.quesName);
        paperDetail.setUnitCount(Integer.parseInt(importPaperCheck.quesCount));
        paperDetail.setNumber(mainQuesNum);
        paperDetail.setCreator(paper.getCreator());
        paperDetails.add(paperDetail);

    }

    /**
     * 初始化试卷明细和试题
     *
     * @param paperDetail
     * @param paperDetailUnit
     * @param question
     * @param subQuesNum
     * @param importPaperCheck
     */
    public void initPaperDetail(Paper paper, PaperDetail paperDetail, PaperDetailUnit paperDetailUnit,
            Question question, int subQuesNum, ImportPaperCheck importPaperCheck) {

        question.setQuestionType(getQuesStructType(importPaperCheck.quesType));
        question.setScore(Double.parseDouble(importPaperCheck.quesScore));
        paperDetailUnit.setNumber(++subQuesNum);
        paperDetailUnit.setQuestion(question);
        paperDetailUnit.setPaperDetail(paperDetail);
        paperDetailUnit.setQuestionType(getQuesStructType(importPaperCheck.quesType));
        paperDetailUnit.setScore(Double.parseDouble(importPaperCheck.quesScore));
        paperDetailUnit.setCreator(paper.getCreator());
        paperDetailUnit.setPaper(paper);
    }

    /**
     * 校验试题头标识
     *
     * @param importPaperCheck
     * @return
     */
    public boolean checkQuesHeader(ImportPaperCheck importPaperCheck) {

        String tmpErrorInfo = "";
        if (StringUtils.isEmpty(importPaperCheck.quesGroup)) {
            tmpErrorInfo += "试题分类为空或格式不正确\n";

        }
        if (StringUtils.isEmpty(importPaperCheck.quesType)) {
            tmpErrorInfo += "题型为空或格式不正确\n";

        }
        if (StringUtils.isEmpty(importPaperCheck.quesName)) {
            tmpErrorInfo += "大题名称为空或格式不正确\n";

        }
        if (StringUtils.isEmpty(importPaperCheck.quesCount)) {
            tmpErrorInfo += "题目数量为空或格式不正确\n";

        }
        if (StringUtils.isEmpty(importPaperCheck.quesScore)) {
            tmpErrorInfo += "分数为空或格式不正确\n";
        }
        if (!StringUtils.isEmpty(importPaperCheck.quesType) && !CommonUtils.checkQuesType(importPaperCheck.quesType)) {
            tmpErrorInfo += "题型必须是单选、多选、判断、填空、问答、套题中的一种\n";
        }
        if (StringUtils.isEmpty(tmpErrorInfo)) {
            return true;
        } else {
            importPaperCheck.setErrorInfo(tmpErrorInfo);
            return false;
        }
    }

    /**
     * 处理大题头信息
     *
     * @param pList
     * @param index
     * @param importPaperCheck
     */
    public void processMainQuesHeader(List pList, int index, ImportPaperCheck importPaperCheck) {

        int i = 0;

        for (i = index; i < pList.size(); i++) {
            P pHeader = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pHeader);
            if (tmpText.startsWith("[试题分类]")) {

                importPaperCheck.setQuesGroup(tmpText.replaceAll("\\[试题分类\\]", "").replaceAll("[:|：]", "").trim());

            } else if (tmpText.startsWith("[题型]")) {

                importPaperCheck.setQuesType(tmpText.replaceAll("\\[题型\\]", "").replaceAll("[:|：]", "").trim());

            } else if (tmpText.startsWith("[大题名称]")) {

                importPaperCheck.setQuesName(tmpText.replaceAll("\\[大题名称\\]", "").replaceAll("[:|：]", "").trim());

            } else if (tmpText.startsWith("[题目数量]")) {

                importPaperCheck.setQuesCount(tmpText.replaceAll("\\[题目数量\\]", "").replaceAll("[:|：]", "").trim());

            } else if (tmpText.startsWith("[分数]")) {

                importPaperCheck.setQuesScore(tmpText.replaceAll("\\[分数\\]", "").replaceAll("[:|：]", "").trim());

            } else if (StringUtils.isEmpty(tmpText)) {
                continue;
            } else {
                break;
            }
        }
        importPaperCheck.setIndex(i);
    }

    /**
     * 处理题干信息
     *
     * @param pList
     * @param index
     * @param question
     * @param importPaperCheck
     * @param wordMLPackage
     * @throws Exception
     */
    public void processQuesBody(List pList, int index, Question question, ImportPaperCheck importPaperCheck,
            WordprocessingMLPackage wordMLPackage) throws Exception {
        // 定义题干wordml和html
        StringBuilder quesBodyWordMl = new StringBuilder();
        StringBuilder quesBodyHtml = new StringBuilder();
        int i = 0;

        for (i = index; i < pList.size(); i++) {
            P pBody = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pBody);

            if (StringUtils.isEmpty(tmpText)) {
                // 跳过空白段落
                continue;
            } else if (tmpText.matches("^\\d{1,}\\.[\\s\\S]*")) {
                // 题干第一段
                // 过滤题干标题
                pBody = DocxProcessUtil.formatP(pBody, QuesUnit.QUES_BODY);
                String tmpWordMl = DocxProcessUtil.getPWordMl(pBody);
                quesBodyWordMl.append(DocxProcessUtil.formatPWordMl(tmpWordMl));
                quesBodyHtml.append(DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
            } else if (tmpText.matches("^[a-zA-Z]\\.[\\s\\S]*") || tmpText.startsWith("[")) {
                // 检测到选项或其他特殊段落直接退出
                break;
            } else {
                // 题干普通段落
                String tmpWordMl = DocxProcessUtil.getPWordMl(pBody);
                quesBodyWordMl.append(DocxProcessUtil.formatPWordMl(tmpWordMl));
                quesBodyHtml.append(DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
            }
        }
        importPaperCheck.setIndex(i);
        question.setQuesBody(quesBodyHtml.toString());
        question.setQuesBodyWord(quesBodyWordMl.toString());
    }

    /**
     * 处理题目选项
     *
     * @param pList
     * @param index
     * @param subQuesNum
     * @param question
     * @param importPaperCheck
     * @param wordMLPackage
     * @return
     * @throws Exception
     */
    public String processQuesOption(List pList, int index, int subQuesNum, Question question,
            ImportPaperCheck importPaperCheck, WordprocessingMLPackage wordMLPackage) throws Exception {

        // 定义试题选项wordml和html
        // String quesOptionWordMl = "";
        // String quesOptionHtml = "";
        // 定义选项集合
        List<QuesOption> quesOptions = new ArrayList<QuesOption>();
        // 定义选项数量
        int optionCount = 0;
        // 当前所处的选项
        QuesOption current = null;

        int i = 0;
        int number = 0;
        for (i = index; i < pList.size(); i++) {
            number++;
            P pOption = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pOption);

            if (StringUtils.isEmpty(tmpText)) {
                // 跳过空白段落
                continue;
            } else if (tmpText.matches("^[a-zA-Z]\\.[\\s\\S]*")) {
                // 检测到选项开始段落
                // 创建试题选项
                current = new QuesOption();
                String optionNumber = String.valueOf(number);// number设置
                                                             // 为1,2,3,4--chenken
                                                             // 2017.4.25
                current.setNumber(optionNumber);
                current.setOptionBody("");
                current.setOptionBodyWord("");
                quesOptions.add(current);
                optionCount++;
                // 过滤选项标题
                pOption = DocxProcessUtil.formatP(pOption, QuesUnit.QUES_OPTION);
                String tmpWordMl = DocxProcessUtil.getPWordMl(pOption);
                current.setOptionBody(current.getOptionBody() + DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
                current.setOptionBodyWord(current.getOptionBodyWord() + DocxProcessUtil.formatPWordMl(tmpWordMl));
            } else if (tmpText.startsWith("[")) {
                // 非选项的其他内容直接退出
                break;
            } else if (current != null) {
                // 选项的其他段落
                String tmpWordMl = DocxProcessUtil.getPWordMl(pOption);
                current.setOptionBody(current.getOptionBody() + DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
                current.setOptionBodyWord(current.getOptionBodyWord() + DocxProcessUtil.formatPWordMl(tmpWordMl));
            }
        }
        importPaperCheck.setIndex(i);
        if (optionCount < 2) {
            importPaperCheck.setErrorInfo(
                    getQuesNumInfo(importPaperCheck.quesName, subQuesNum) + "中选项格式不正确或有缺失,如没有缺失可尝试换行处理\n");
        } else {
            question.setQuesOptions(quesOptions);
        }
        return importPaperCheck.errorInfo;
    }

    /**
     * 处理小题尾信息
     *
     * @param pList
     * @param index
     * @param subQuesNum
     * @param question
     * @param paperDetailUnit
     * @param importPaperCheck
     */
    public void processQuesTail(List pList, int index, int subQuesNum, Question question,
            PaperDetailUnit paperDetailUnit, ImportPaperCheck importPaperCheck, WordprocessingMLPackage wordMLPackage,
            boolean isNested) throws Exception {

        StringBuilder answerWordML = new StringBuilder("");
        StringBuilder answerHTML = new StringBuilder("");
        String quesScore = "";
        // Map<String, String> quesParams = new HashMap<String, String>();
        int i = 0;
        // 是否刚刚检测到答案内容
        boolean answerStart = false;

        for (i = index; i < pList.size(); i++) {
            P pAnswer = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pAnswer);
            if (StringUtils.isEmpty(tmpText)) {
                // 跳过空白段落
                continue;
            } else if (tmpText.startsWith("[答案]")) {
                // 检测到答案开始段落
                pAnswer = DocxProcessUtil.formatP(pAnswer, QuesUnit.QUES_ANSWER);
                String tmpWordMl = DocxProcessUtil.getPWordMl(pAnswer);
                answerWordML.append(DocxProcessUtil.formatPWordMl(tmpWordMl));
                answerHTML.append(DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
                answerStart = true;
            } else if (tmpText.startsWith("[小题分数]")) {
                // 检测到分数开始段落
                quesScore = tmpText.replaceAll("\\[小题分数\\]", "").replaceAll("[:|：]", "").trim();
                answerStart = false;
            } else if (tmpText.startsWith("[") || tmpText.matches("^\\d{1,}\\.[\\s\\S]*")) {
                // 检测到其他特殊段落或下一题直接退出
                break;
            } else if (answerStart) {
                String tmpWordMl = DocxProcessUtil.getPWordMl(pAnswer);
                answerWordML.append(DocxProcessUtil.formatPWordMl(tmpWordMl));
                answerHTML.append(DocxProcessUtil.docx2Html(tmpWordMl, wordMLPackage));
            }
        }
        importPaperCheck.setIndex(i);
        // 校验答案
        if (answerHTML.length() > 0) {
            // 选择题过滤html标签--chenken 2017.4.25
            if (question.getQuestionType() == QuesStructType.SINGLE_ANSWER_QUESTION
                    || question.getQuestionType() == QuesStructType.MULTIPLE_ANSWER_QUESTION
                    || question.getQuestionType() == QuesStructType.BOOL_ANSWER_QUESTION) {
                String regEx_html = "<[^>]+>";
                Pattern p_html = Pattern.compile(regEx_html, Pattern.CASE_INSENSITIVE);
                Matcher m_html = p_html.matcher(answerHTML.toString()); // 过滤html标签
                question.setQuesAnswer(m_html.replaceAll(""));
            } else {
                question.setQuesAnswer(answerHTML.toString());
            }
            question.setQuesAnswerWord(answerWordML.toString());
        } else {
            importPaperCheck.setErrorInfo(getQuesNumInfo(importPaperCheck.quesName, subQuesNum) + "答案为空或格式不正确\n");
        }

        // 设置预设分数
        if (StringUtils.isNumeric(quesScore)) {
            question.setScore(Double.parseDouble(quesScore));
        }
        // 一般大题明细需要设置分数
        if (!isNested && StringUtils.isNumeric(quesScore)) {
            paperDetailUnit.setScore(Double.parseDouble(quesScore));
        }
    }

    /**
     * 处理选择题option信息
     * 
     * @param question
     */
    public void processSelectOption(Question question) {
        String answer = question.getQuesAnswer();
        if (StringUtils.isNotBlank(answer)) {
            String[] answerArray = answer.split(",");
            for (int i = 0; i < question.getQuesOptions().size(); i++) {
                QuesOption quesOption = question.getQuesOptions().get(i);
                char number = (char) (Integer.parseInt(quesOption.getNumber()) + 64);
                if (ArrayUtils.contains(answerArray, String.valueOf(number))) {
                    quesOption.setIsCorrect((short) 1);
                } else {
                    quesOption.setIsCorrect((short) 0);
                }
            }
            // 选择题答案不持久化
            question.setQuesAnswer(null);
        }
    }

    /**
     * 处理套题
     *
     * @param pList
     * @param index
     * @param question
     * @param paperDetailUnit
     * @param importPaperCheck
     * @throws Exception
     */
    public void processNestedQues(List pList, int index, Question question, PaperDetailUnit paperDetailUnit,
            ImportPaperCheck importPaperCheck, WordprocessingMLPackage wordMLPackage) throws Exception {
        // 题型
        String nestedQuesType = "";

        // 设置套题题干
        processQuesBody(pList, index, question, importPaperCheck, wordMLPackage);

        // 创建小题集合
        List<Question> subQuesList = new ArrayList<Question>();

        int beginNum = importPaperCheck.index;

        int subQuesNum = 0;
        for (int i = beginNum; i < pList.size(); i++) {
            P pSubQues = (P) pList.get(i);
            String tmpText = DocxProcessUtil.getPText(pSubQues);

            if (StringUtils.isEmpty(tmpText)) {
                continue;
            }
            if (tmpText.startsWith("[小题型]")) {

                nestedQuesType = tmpText.replaceAll("\\[小题型\\]", "").replaceAll("[:|：]", "").trim();
                importPaperCheck.index++;

            } else if (tmpText.matches("^\\d\\.[\\s\\S]*")) {

                ++subQuesNum;

                if (StringUtils.isEmpty(nestedQuesType)) {
                    importPaperCheck
                            .setErrorInfo(getQuesNumInfo(importPaperCheck.quesName, subQuesNum) + "小题型为空或格式不正确\n");
                    return;
                }

                Question subQues = new Question();
                subQues.setQuestionType(getQuesStructType(nestedQuesType));
                if (StringUtils.isNumeric(importPaperCheck.getQuesScore())) {
                    subQues.setScore(Double.parseDouble(importPaperCheck.getQuesScore()));
                }

                // 处理客观题
                if (nestedQuesType.equals("单选") || nestedQuesType.equals("多选")) {

                    // 处理题干
                    processQuesBody(pList, importPaperCheck.index, subQues, importPaperCheck, wordMLPackage);

                    // 处理选项
                    processQuesOption(pList, importPaperCheck.index, subQuesNum, subQues, importPaperCheck,
                            wordMLPackage);

                    // 处理尾信息
                    processQuesTail(pList, importPaperCheck.index, subQuesNum, subQues, paperDetailUnit,
                            importPaperCheck, wordMLPackage, true);

                    // 处理选择题的option--chenken 20170426
                    processSelectOption(subQues);

                } else {
                    // 处理其他题型
                    processQuesBody(pList, importPaperCheck.index, subQues, importPaperCheck, wordMLPackage);

                    processQuesTail(pList, importPaperCheck.index, subQuesNum, subQues, paperDetailUnit,
                            importPaperCheck, wordMLPackage, true);
                }
                subQuesList.add(subQues);
                i = importPaperCheck.index - 1;
            } else {
                break;
            }
            if (!StringUtils.isEmpty(importPaperCheck.errorInfo)) {
                return;
            }
        }
        question.setSubQuestions(subQuesList);
        // 计算套题总分
        double totalScore = 0;
        List<Double> scoreList = new ArrayList<>();
        for (Question child : subQuesList) {
            Double score = child.getScore();
            if (score != null) {
                totalScore += score;
                scoreList.add(score);
            } else {
                scoreList.add(0d);
            }
        }
        question.setScore(totalScore);
        paperDetailUnit.setSubScoreList(scoreList);
    }

    /**
     * 获取试题类型
     *
     * @param quesType
     * @return
     */
    public static QuestionType getQuesType(String quesType) {
        return getQuesType(quesType, "");
    }

    /**
     * 获取试题类型
     *
     * @param quesType
     * @param quesName
     * @return
     */
    public static QuestionType getQuesType(String quesType, String quesName) {
        QuesStructType quesStructType = CommonUtils.getEnum(QuesStructType.class, quesType);
        return new QuestionType(quesStructType, quesName);
    }

    /**
     * 获取试题结构类型
     *
     * @param quesType
     * @return
     */
    public static QuesStructType getQuesStructType(String quesType) {
        return CommonUtils.getEnum(QuesStructType.class, quesType);
    }

    /**
     * 获取当前试题大题小题号
     *
     * @param quesType
     * @param subQuesNum
     * @return
     */
    public static String getQuesNumInfo(String quesType, int subQuesNum) {
        return quesType + "第" + subQuesNum + "小题";
    }

    /**
     * 判断是否为套题
     *
     * @param importPaperCheck
     * @return
     */
    public static boolean isNested(ImportPaperCheck importPaperCheck) {
        if (importPaperCheck.getQuesType().equals("套题")) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 获取课程代码
     *
     * @param importPaperCheck
     * @return
     */
    public static String getCourseNo(ImportPaperCheck importPaperCheck) {
        String[] courseInfo = importPaperCheck.quesGroup.split("_");
        if (courseInfo.length == 2) {
            return courseInfo[1];
        } else {
            return "";
        }
    }

    /**
     * 获取课程名称
     *
     * @param importPaperCheck
     * @return
     */
    public static String getCourseName(ImportPaperCheck importPaperCheck) {
        String[] courseInfo = importPaperCheck.quesGroup.split("_");
        if (courseInfo.length == 2) {
            return courseInfo[0];
        } else {
            return "";
        }
    }

    /**
     * 设置wordMLPackage对象二进制数据
     *
     * @param question
     * @param wordMLPackage
     * @throws Exception
     */
    public static void setPkgByte(Question question, WordprocessingMLPackage wordMLPackage) throws Exception {
        question.setQuesPkg(DocxProcessUtil.getPkgByte(wordMLPackage));
    }

    /**
     * 构造一张空白的导入类型试卷
     *
     * @param courseNo
     * @param paperName
     * @return
     */
    public Map<String, Object> saveBlankPaper(String courseNo, String courseName, String paperName, AccessUser user) {
        Map<String, Object> returnMap = new HashMap<String, Object>();
        Paper paper = new Paper();
        String msg = paperService.checkPaperName(paperName, user.getOrgId().toString());
        if (msg != null) {
            returnMap.put("msg", msg);
            return returnMap;
        }
        initPaper(paper, paperName);
        paper.setCourseNo(courseNo);
        paper.setCourseName(courseName);
        paper.setOrgId(user.getOrgId().toString());
        paper.setCreator(user.getLoginName());
        paper.setCreateTime(CommonUtils.getCurDateTime());
        PaperDetail pd = new PaperDetail();
        pd.setId(UUID.randomUUID().toString());
        pd.setNumber(1);
        pd.setPaper(paper);
        pd.setCreator(user.getLoginName());
        pd.setCreateTime(CommonUtils.getCurDateTime());
        paper = paperRepo.save(paper);
        paperDetailRepo.save(pd);
        returnMap.put("paper", paper);
        returnMap.put("msg", "success");
        return returnMap;
    }
}
